from .settings import PythonSettings, python_settings
from .tasks.black_task import BlackTask, black
from .tasks.build_task import BuildTask, build
from .tasks.flake8_task import Flake8Task, flake8
from .tasks.install_task import InstallTask, install
from .tasks.isort_task import IsortTask, isort
from .tasks.mypy_task import MypyTask, mypy
from .tasks.publish_task import PublishTask, publish
from .tasks.pytest_task import PytestTask, pytest

# Backwards compatibilty
from .version import git_version_to_python_version, git_version_to_python_version as git_version_to_python

__all__ = [
    "PythonSettings",
    "python_settings",
    "BlackTask",
    "black",
    "BuildTask",
    "build",
    "InstallTask",
    "install",
    "IsortTask",
    "isort",
    "Flake8Task",
    "flake8",
    "MypyTask",
    "mypy",
    "PublishTask",
    "publish",
    "PytestTask",
    "pytest",
    "git_version_to_python_version",
    "git_version_to_python",
]
